#include <QPushButton>
#include <QLayout>
#include <QTimer>
#include <QPaintEvent>
#include <QResizeEvent>
#include <QMouseEvent>
#include <QShowEvent>

#include <stdlib.h>

#include <TCanvas.h>
#include <TVirtualX.h>
#include <TSystem.h>
#include <TApplication.h>
#include <TFormula.h>
#include <TF1.h>
#include <TH1.h>
#include <TFrame.h>
#include <TTimer.h>
#include "canvas.h"


//--------------------------------------------------------------------------------

QRootCanvas::QRootCanvas(QWidget *parent) :
             QWidget(parent, 0), fCanvas(0)
{
   setMinimumSize(600, 500);
   setMouseTracking(kTRUE);

   int wid = gVirtualX->AddWindow((ULong_t)winId(),600,400);
   fCanvas = new TCanvas("Root Canvas",width(),height(),wid);
}

void QRootCanvas::mouseMoveEvent(QMouseEvent *e)
{
   if (fCanvas) {
      if (e->buttons() & Qt::LeftButton) {
         fCanvas->HandleInput(kButton1Motion, e->x(), e->y());
      } else if (e->buttons() & Qt::MidButton) {
         fCanvas->HandleInput(kButton2Motion, e->x(), e->y());
      } else if (e->buttons() & Qt::RightButton) {
         fCanvas->HandleInput(kButton3Motion, e->x(), e->y());
      } else {
         fCanvas->HandleInput(kMouseMotion, e->x(), e->y());
      }
   }
}

void QRootCanvas::mousePressEvent( QMouseEvent *e )
{
   if (fCanvas) {
      switch (e->button()) {
         case Qt::LeftButton :
            fCanvas->HandleInput(kButton1Down, e->x(), e->y());
            break;
         case Qt::MidButton :
            fCanvas->HandleInput(kButton2Down, e->x(), e->y());
            break;
         case Qt::RightButton :
            // does not work properly on Linux...
#ifdef WIN32
            fCanvas->HandleInput(kButton3Down, e->x(), e->y());
#endif
            break;
         default:
            break;
      }
   }
}

void QRootCanvas::mouseReleaseEvent( QMouseEvent *e )
{
   if (fCanvas) {
      switch (e->button()) {
         case Qt::LeftButton :
            fCanvas->HandleInput(kButton1Up, e->x(), e->y());
            break;
         case Qt::MidButton :
            fCanvas->HandleInput(kButton2Up, e->x(), e->y());
            break;
         case Qt::RightButton :
            // does not work properly on Linux...
#ifdef WIN32
            fCanvas->HandleInput(kButton3Up, e->x(), e->y());
#endif
            break;
         default:
            break;
      }
   }
}

void QRootCanvas::resizeEvent( QResizeEvent * )
{
   if (fCanvas) {
      fCanvas->Resize();
      fCanvas->Update();
   }
}

void QRootCanvas::paintEvent( QPaintEvent * )
{
   if (fCanvas) {
      fCanvas->Resize();
      fCanvas->Update();
   }
}

QMainCanvas::QMainCanvas(QWidget *parent) : QWidget(parent)
{

   QVBoxLayout *l = new QVBoxLayout(this);
   l->addWidget(canvas = new QRootCanvas(this));
   l->addWidget(b = new QPushButton("&Draw Histogram", this));
   connect(b, SIGNAL(clicked()), this, SLOT(clicked1()));
   fRootTimer = new QTimer( this );
   QObject::connect( fRootTimer, SIGNAL(timeout()), this, SLOT(handle_root_events()) );
   fRootTimer->start( 20 );
}

void QMainCanvas::clicked1()
{
   TFormula *form1 = new TFormula("form1","abs(sin(x)/x)");
   TF1 *sqroot = new TF1("sqroot","x*gaus(0) + [3]*form1",0,10);
   sqroot->SetParameters(10,4,1,20);

   // hack to keep the histogram displayed on the canvas while resizing
   // waiting for any Qt expert to fix this...
   canvas->setUpdatesEnabled(kFALSE);
   //
   // Create a one dimensional histogram (one float per bin)
   // and fill it following the distribution in function sqroot.
   //
   canvas->getCanvas()->cd();
   canvas->getCanvas()->GetFrame()->SetFillColor(42);
   canvas->getCanvas()->GetFrame()->SetBorderMode(-1);
   canvas->getCanvas()->GetFrame()->SetBorderSize(0);
   TH1F *h1f = new TH1F("h1f","Test random numbers",200,0,10);
   h1f->SetFillColor(45);
   h1f->FillRandom("sqroot",10000);
   h1f->Draw();
   canvas->getCanvas()->Modified();
   canvas->getCanvas()->Update();
}

//--------------------------------------------------------------------------------

void QMainCanvas::handle_root_events()
{
   //call the inner loop of ROOT
   gSystem->ProcessEvents();
}


